@@ -22,49 +22,28 @@
 #include "soup-uri-utils-private.h"
 
 /**
- * SECTION:soup-auth-manager
- * @short_description: HTTP client-side authentication handler
- * @see_also: #SoupSession, #SoupAuth
+ * SoupAuthManager:
  *
- * #SoupAuthManager is the #SoupSessionFeature that handles HTTP
- * authentication for a #SoupSession.
+ * HTTP client-side authentication handler.
+ *
+ * #SoupAuthManager is the [iface@SessionFeature] that handles HTTP
+ * authentication for a [class@Session].
  *
  * A #SoupAuthManager is added to the session by default, and normally
  * you don't need to worry about it at all. However, if you want to
  * disable HTTP authentication, you can remove the feature from the
- * session with soup_session_remove_feature_by_type(), or disable it on
- * individual requests with soup_message_disable_feature().
+ * session with [method@Session.remove_feature_by_type] or disable it on
+ * individual requests with [method@Message.disable_feature].
  *
+ * You can use this with [method@Session.remove_feature_by_type] or
+ * [method@Message.disable_feature].
+ *
+ * (Although this type has only been publicly visible since libsoup 2.42, it has
+ * always existed in the background, and you can use `g_type_from_name
+ * ("SoupAuthManager")` to get its [alias@GObject.Type] in earlier releases.)
  **/
-
-/**
- * SoupAuthManager:
- *
- * Class for managing client-side HTTP authentication.
- */
-
-/**
- * SOUP_TYPE_AUTH_MANAGER:
- *
- * The #GType of #SoupAuthManager; you can use this with
- * soup_session_remove_feature_by_type() or
- * soup_message_disable_feature().
- *
- * (Although this type has only been publicly visible since libsoup
- * 2.42, it has always existed in the background, and you can use
- * <literal><code>g_type_from_name ("SoupAuthManager")</code></literal>
- * to get its #GType in earlier releases.)
- *
- */
 static void soup_auth_manager_session_feature_init (SoupSessionFeatureInterface *feature_interface, gpointer interface_data);
 
-enum {
-	AUTHENTICATE,
-	LAST_SIGNAL
-};
-
-static guint signals[LAST_SIGNAL] = { 0 };
-
 struct _SoupAuthManager {
         GObject parent_instance;
 };
@@ -75,6 +54,7 @@ typedef struct {
 	gboolean auto_ntlm;
 
 	SoupAuth *proxy_auth;
+        GMutex mutex;
 	GHashTable *auth_hosts;
 } SoupAuthManagerPrivate;
 
@@ -104,6 +84,7 @@ soup_auth_manager_init (SoupAuthManager *manager)
 						  soup_uri_host_equal,
 						  NULL,
 						  (GDestroyNotify)soup_auth_host_free);
+        g_mutex_init (&priv->mutex);
 }
 
 static void
@@ -117,6 +98,8 @@ soup_auth_manager_finalize (GObject *object)
 
 	g_clear_object (&priv->proxy_auth);
 
+        g_mutex_clear (&priv->mutex);
+
 	G_OBJECT_CLASS (soup_auth_manager_parent_class)->finalize (object);
 }
 
@@ -126,33 +109,6 @@ soup_auth_manager_class_init (SoupAuthManagerClass *auth_manager_class)
 	GObjectClass *object_class = G_OBJECT_CLASS (auth_manager_class);
 
 	object_class->finalize = soup_auth_manager_finalize;
-
-	/**
-	 * SoupAuthManager::authenticate:
-	 * @manager: the #SoupAuthManager
-	 * @msg: the #SoupMessage being sent
-	 * @auth: the #SoupAuth to authenticate
-	 * @retrying: %TRUE if this is the second (or later) attempt
-	 *
-	 * Emitted when the manager requires the application to
-	 * provide authentication credentials.
-	 *
-	 * #SoupMessage connects to this signal and emits its own
-	 * #SoupMessage::authenticate signal when it is emitted, so
-	 * you shouldn't need to use this signal directly.
-	 */
-	signals[AUTHENTICATE] =
-		g_signal_new ("authenticate",
-			      G_OBJECT_CLASS_TYPE (object_class),
-			      G_SIGNAL_RUN_FIRST,
-			      0,
-			      NULL, NULL,
-			      NULL,
-			      G_TYPE_NONE, 3,
-			      SOUP_TYPE_MESSAGE,
-			      SOUP_TYPE_AUTH,
-			      G_TYPE_BOOLEAN);
-
 }
 
 static int
@@ -664,6 +620,8 @@ auth_got_headers (SoupMessage *msg, gpointer manager)
 	SoupAuth *auth, *prior_auth;
 	gboolean prior_auth_failed = FALSE;
 
+        g_mutex_lock (&priv->mutex);
+
 	/* See if we used auth last time */
 	prior_auth = soup_message_get_auth (msg);
 	if (prior_auth && check_auth (msg, prior_auth)) {
@@ -672,8 +630,10 @@ auth_got_headers (SoupMessage *msg, gpointer manager)
 			prior_auth_failed = TRUE;
 	} else {
 		auth = create_auth (priv, msg);
-		if (!auth)
+		if (!auth) {
+                        g_mutex_unlock (&priv->mutex);
 			return;
+                }
 	}
 
 	if (!soup_message_query_flags (msg, SOUP_MESSAGE_DO_NOT_USE_AUTH_CACHE)) {
@@ -685,6 +645,8 @@ auth_got_headers (SoupMessage *msg, gpointer manager)
 		auth = g_object_ref (new_auth);
 	}
 
+        g_mutex_unlock (&priv->mutex);
+
 	/* If we need to authenticate, try to do it. */
 	authenticate_auth (manager, auth, msg,
 			   prior_auth_failed, FALSE, TRUE);
@@ -698,6 +660,8 @@ auth_got_body (SoupMessage *msg, gpointer manager)
         SoupAuthManagerPrivate *priv = soup_auth_manager_get_instance_private (manager);
 	SoupAuth *auth;
 
+        g_mutex_lock (&priv->mutex);
+
 	auth = lookup_auth (priv, msg);
 	if (auth && soup_auth_is_ready (auth, msg)) {
 		if (SOUP_IS_CONNECTION_AUTH (auth))
@@ -711,6 +675,8 @@ auth_got_body (SoupMessage *msg, gpointer manager)
 
 		soup_session_requeue_message (priv->session, msg);
 	}
+
+        g_mutex_unlock (&priv->mutex);
 }
 
 static void
@@ -720,6 +686,8 @@ proxy_auth_got_headers (SoupMessage *msg, gpointer manager)
 	SoupAuth *auth = NULL, *prior_auth;
 	gboolean prior_auth_failed = FALSE;
 
+        g_mutex_lock (&priv->mutex);
+
 	/* See if we used auth last time */
 	prior_auth = soup_message_get_proxy_auth (msg);
 	if (prior_auth && check_auth (msg, prior_auth)) {
@@ -732,13 +700,17 @@ proxy_auth_got_headers (SoupMessage *msg, gpointer manager)
 
 	if (!auth) {
 		auth = create_auth (priv, msg);
-		if (!auth)
+		if (!auth) {
+                        g_mutex_unlock (&priv->mutex);
 			return;
+                }
 
 		if (!soup_message_query_flags (msg, SOUP_MESSAGE_DO_NOT_USE_AUTH_CACHE))
 			priv->proxy_auth = g_object_ref (auth);
 	}
 
+        g_mutex_unlock (&priv->mutex);
+
 	/* If we need to authenticate, try to do it. */
 	authenticate_auth (manager, auth, msg,
 			   prior_auth_failed, TRUE, TRUE);
@@ -752,6 +724,8 @@ proxy_auth_got_body (SoupMessage *msg, gpointer manager)
         SoupAuthManagerPrivate *priv = soup_auth_manager_get_instance_private (manager);
 	SoupAuth *auth;
 
+        g_mutex_lock (&priv->mutex);
+
 	auth = lookup_proxy_auth (priv, msg);
 	if (auth && soup_auth_is_ready (auth, msg)) {
 		/* When not using cached credentials, update the Authorization header
@@ -761,6 +735,8 @@ proxy_auth_got_body (SoupMessage *msg, gpointer manager)
 			update_authorization_header (msg, auth, TRUE);
 		soup_session_requeue_message (priv->session, msg);
 	}
+
+        g_mutex_unlock (&priv->mutex);
 }
 
 static void
@@ -772,6 +748,8 @@ auth_msg_starting (SoupMessage *msg, gpointer manager)
 	if (soup_message_query_flags (msg, SOUP_MESSAGE_DO_NOT_USE_AUTH_CACHE))
 		return;
 
+        g_mutex_lock (&priv->mutex);
+
 	if (soup_message_get_method (msg) != SOUP_METHOD_CONNECT) {
 		auth = lookup_auth (priv, msg);
 		if (auth) {
@@ -791,6 +769,8 @@ auth_msg_starting (SoupMessage *msg, gpointer manager)
 	}
 	soup_message_set_proxy_auth (msg, auth);
 	update_authorization_header (msg, auth, TRUE);
+
+        g_mutex_unlock (&priv->mutex);
 }
 
 static void
@@ -830,10 +810,11 @@ soup_auth_manager_request_unqueued (SoupSessionFeature *manager,
  * @auth: the #SoupAuth to use
  *
  * Records that @auth is to be used under @uri, as though a
- * WWW-Authenticate header had been received at that URI. This can be
- * used to "preload" @manager's auth cache, to avoid an extra HTTP
- * round trip in the case where you know ahead of time that a 401
- * response will be returned.
+ * WWW-Authenticate header had been received at that URI.
+ *
+ * This can be used to "preload" @manager's auth cache, to avoid an extra HTTP
+ * round trip in the case where you know ahead of time that a 401 response will
+ * be returned.
  *
  * This is only useful for authentication types where the initial
  * Authorization header does not depend on any additional information
@@ -847,14 +828,16 @@ soup_auth_manager_use_auth (SoupAuthManager *manager,
 {
         SoupAuthManagerPrivate *priv = soup_auth_manager_get_instance_private (manager);
 
+        g_mutex_lock (&priv->mutex);
 	record_auth_for_uri (priv, uri, auth, FALSE);
+        g_mutex_unlock (&priv->mutex);
 }
 
 /**
  * soup_auth_manager_clear_cached_credentials:
  * @manager: a #SoupAuthManager
  *
- * Clear all credentials cached by @manager
+ * Clear all credentials cached by @manager.
  *
  */
 void
@@ -864,7 +847,9 @@ soup_auth_manager_clear_cached_credentials (SoupAuthManager *manager)
 
 	g_return_if_fail (SOUP_IS_AUTH_MANAGER (manager));
 
+        g_mutex_lock (&priv->mutex);
 	g_hash_table_remove_all (priv->auth_hosts);
+        g_mutex_unlock (&priv->mutex);
 }
 
 static void
