@@ -19,39 +19,32 @@
 #include "soup-uri-utils-private.h"
 
 /**
- * SECTION:soup-hsts-enforcer
- * @short_description: Automatic HTTP Strict Transport Security enforcing
- * for #SoupSession
+ * SoupHSTSEnforcer:
+ *
+ * Automatic HTTP Strict Transport Security enforcing for [class@Session].
  *
  * A #SoupHSTSEnforcer stores HSTS policies and enforces them when
- * required. #SoupHSTSEnforcer implements #SoupSessionFeature, so you
+ * required. #SoupHSTSEnforcer implements [iface@SessionFeature], so you
  * can add an HSTS enforcer to a session with
- * soup_session_add_feature() or soup_session_add_feature_by_type().
+ * [method@Session.add_feature] or [method@Session.add_feature_by_type].
  *
  * #SoupHSTSEnforcer keeps track of all the HTTPS destinations that,
  * when connected to, return the Strict-Transport-Security header with
  * valid values. #SoupHSTSEnforcer will forget those destinations
  * upon expiry or when the server requests it.
  *
- * When the #SoupSession the #SoupHSTSEnforcer is attached to queues
- * or restarts a message, the #SoupHSTSEnforcer will rewrite the URI
- * to HTTPS if the destination is a known HSTS host and is contacted
- * over an insecure transport protocol (HTTP). Users of
- * #SoupHSTSEnforcer are advised to listen to changes in
- * SoupMessage:uri in order to be aware of changes in the message URI.
+ * When the [class@Session] the #SoupHSTSEnforcer is attached to queues or
+ * restarts a message, the #SoupHSTSEnforcer will rewrite the URI to HTTPS if
+ * the destination is a known HSTS host and is contacted over an insecure
+ * transport protocol (HTTP). Users of #SoupHSTSEnforcer are advised to listen
+ * to changes in the [property@Message:uri] property in order to be aware of
+ * changes in the message URI.
  *
  * Note that #SoupHSTSEnforcer does not support any form of long-term
- * HSTS policy persistence. See #SoupHSTSEnforcerDB for a persistent
+ * HSTS policy persistence. See [class@HSTSEnforcerDB] for a persistent
  * enforcer.
- *
  **/
 
-/**
- * SoupHSTSEnforcer:
- *
- * Class for storing and enforcing a #SoupHSTSPolicy.
- */
-
 static void soup_hsts_enforcer_session_feature_init (SoupSessionFeatureInterface *feature_interface, gpointer interface_data);
 
 enum {
@@ -63,6 +56,7 @@ static guint signals[LAST_SIGNAL] = { 0 };
 
 typedef struct {
 	SoupSession *session;
+        GMutex mutex;
 	GHashTable *host_policies;
 	GHashTable *session_policies;
 } SoupHSTSEnforcerPrivate;
@@ -79,30 +73,24 @@ soup_hsts_enforcer_init (SoupHSTSEnforcer *hsts_enforcer)
 
 	priv->host_policies = g_hash_table_new_full (soup_str_case_hash,
 								    soup_str_case_equal,
-								    g_free, NULL);
+								    g_free, (GDestroyNotify)soup_hsts_policy_free);
 
 	priv->session_policies = g_hash_table_new_full (soup_str_case_hash,
 								       soup_str_case_equal,
-								       g_free, NULL);
+								       g_free, (GDestroyNotify)soup_hsts_policy_free);
+        g_mutex_init (&priv->mutex);
 }
 
 static void
 soup_hsts_enforcer_finalize (GObject *object)
 {
 	SoupHSTSEnforcerPrivate *priv = soup_hsts_enforcer_get_instance_private ((SoupHSTSEnforcer*)object);
-	GHashTableIter iter;
-	gpointer key, value;
 
-	g_hash_table_iter_init (&iter, priv->host_policies);
-	while (g_hash_table_iter_next (&iter, &key, &value))
-		soup_hsts_policy_free (value);
 	g_hash_table_destroy (priv->host_policies);
-
-	g_hash_table_iter_init (&iter, priv->session_policies);
-	while (g_hash_table_iter_next (&iter, &key, &value))
-		soup_hsts_policy_free (value);
 	g_hash_table_destroy (priv->session_policies);
 
+        g_mutex_clear (&priv->mutex);
+
 	G_OBJECT_CLASS (soup_hsts_enforcer_parent_class)->finalize (object);
 }
 
@@ -160,7 +148,9 @@ soup_hsts_enforcer_class_init (SoupHSTSEnforcerClass *hsts_enforcer_class)
 	 * @old_policy: the old #SoupHSTSPolicy value
 	 * @new_policy: the new #SoupHSTSPolicy value
 	 *
-	 * Emitted when @hsts_enforcer changes. If a policy has been added,
+	 * Emitted when @hsts_enforcer changes.
+	 *
+	 * If a policy has been added,
 	 * @new_policy will contain the newly-added policy and
 	 * @old_policy will be %NULL. If a policy has been deleted,
 	 * @old_policy will contain the to-be-deleted policy and
@@ -186,12 +176,12 @@ soup_hsts_enforcer_class_init (SoupHSTSEnforcerClass *hsts_enforcer_class)
 /**
  * soup_hsts_enforcer_new:
  *
- * Creates a new #SoupHSTSEnforcer. The base #SoupHSTSEnforcer class
- * does not support persistent storage of HSTS policies, see
- * #SoupHSTSEnforcerDB for that.
+ * Creates a new #SoupHSTSEnforcer.
+ *
+ * The base #SoupHSTSEnforcer class does not support persistent storage of HSTS
+ * policies, see [class@HSTSEnforcerDB] for that.
  *
  * Returns: a new #SoupHSTSEnforcer
- *
  **/
 SoupHSTSEnforcer *
 soup_hsts_enforcer_new (void)
@@ -219,8 +209,6 @@ should_remove_expired_host_policy (G_GNUC_UNUSED gpointer key,
 		   table, which could be problematic, or not.
 		*/
 		soup_hsts_enforcer_changed (enforcer, policy, NULL);
-		soup_hsts_policy_free (policy);
-
 		return TRUE;
 	}
 
@@ -242,15 +230,14 @@ soup_hsts_enforcer_remove_host_policy (SoupHSTSEnforcer *hsts_enforcer,
 {
         SoupHSTSEnforcerPrivate *priv = soup_hsts_enforcer_get_instance_private (hsts_enforcer);
 	SoupHSTSPolicy *policy;
+        char *key;
 
-	policy = g_hash_table_lookup (priv->host_policies, domain);
+        if (!g_hash_table_steal_extended (priv->host_policies, domain, (gpointer*)&key, (gpointer*)&policy))
+                return;
 
-	if (!policy)
-		return;
-
-	g_hash_table_remove (priv->host_policies, domain);
 	soup_hsts_enforcer_changed (hsts_enforcer, policy, NULL);
 	soup_hsts_policy_free (policy);
+        g_free (key);
 
 	remove_expired_host_policies (hsts_enforcer);
 }
@@ -263,6 +250,7 @@ soup_hsts_enforcer_replace_policy (SoupHSTSEnforcer *hsts_enforcer,
 	GHashTable *policies;
 	SoupHSTSPolicy *old_policy;
 	const char *domain;
+        char *old_key;
 	gboolean is_session_policy;
 
 	g_assert (!soup_hsts_policy_is_expired (new_policy));
@@ -273,10 +261,10 @@ soup_hsts_enforcer_replace_policy (SoupHSTSEnforcer *hsts_enforcer,
 	policies = is_session_policy ? priv->session_policies :
 		                       priv->host_policies;
 
-	old_policy = g_hash_table_lookup (policies, domain);
-	g_assert (old_policy);
+        g_assert (g_hash_table_steal_extended (policies, domain,
+                                              (gpointer*)&old_key, (gpointer*)&old_policy));
 
-	g_hash_table_replace (policies, g_strdup (domain), soup_hsts_policy_copy (new_policy));
+	g_hash_table_insert (policies, g_steal_pointer (&old_key), soup_hsts_policy_copy (new_policy));
 	if (!soup_hsts_policy_equal (old_policy, new_policy))
 		soup_hsts_enforcer_changed (hsts_enforcer, old_policy, new_policy);
 	soup_hsts_policy_free (old_policy);
@@ -317,14 +305,14 @@ soup_hsts_enforcer_insert_policy (SoupHSTSEnforcer *hsts_enforcer,
  * @hsts_enforcer: a #SoupHSTSEnforcer
  * @policy: (transfer none): the policy of the HSTS host
  *
- * Sets @policy to @hsts_enforcer. If @policy is expired, any
- * existing HSTS policy for its host will be removed instead. If a
- * policy existed for this host, it will be replaced. Otherwise, the
- * new policy will be inserted. If the policy is a session policy, that
- * is, one created with soup_hsts_policy_new_session_policy(), the policy
- * will not expire and will be enforced during the lifetime of
- * @hsts_enforcer's #SoupSession.
+ * Sets @policy to @hsts_enforcer.
  *
+ * If @policy is expired, any existing HSTS policy for its host will be removed
+ * instead. If a policy existed for this host, it will be replaced. Otherwise,
+ * the new policy will be inserted. If the policy is a session policy, that is,
+ * one created with [ctor@HSTSPolicy.new_session_policy], the policy will not
+ * expire and will be enforced during the lifetime of @hsts_enforcer's
+ * [class@Session].
  **/
 void
 soup_hsts_enforcer_set_policy (SoupHSTSEnforcer *hsts_enforcer,
@@ -342,21 +330,24 @@ soup_hsts_enforcer_set_policy (SoupHSTSEnforcer *hsts_enforcer,
 	domain = soup_hsts_policy_get_domain (policy);
 	g_return_if_fail (domain != NULL);
 
-	is_session_policy = soup_hsts_policy_is_session_policy (policy);
-	policies = is_session_policy ? priv->session_policies :
-				  priv->host_policies;
+        g_mutex_lock (&priv->mutex);
 
+	is_session_policy = soup_hsts_policy_is_session_policy (policy);
 	if (!is_session_policy && soup_hsts_policy_is_expired (policy)) {
 		soup_hsts_enforcer_remove_host_policy (hsts_enforcer, domain);
+                g_mutex_unlock (&priv->mutex);
 		return;
 	}
 
+        policies = is_session_policy ? priv->session_policies : priv->host_policies;
 	current_policy = g_hash_table_lookup (policies, domain);
 
 	if (current_policy)
 		soup_hsts_enforcer_replace_policy (hsts_enforcer, policy);
 	else
 		soup_hsts_enforcer_insert_policy (hsts_enforcer, policy);
+
+        g_mutex_unlock (&priv->mutex);
 }
 
 /**
@@ -365,10 +356,10 @@ soup_hsts_enforcer_set_policy (SoupHSTSEnforcer *hsts_enforcer,
  * @domain: policy domain or hostname
  * @include_subdomains: %TRUE if the policy applies on sub domains
  *
- * Sets a session policy for @domain. A session policy is a policy
- * that is permanent to the lifetime of @hsts_enforcer's #SoupSession
- * and doesn't expire.
+ * Sets a session policy for @domain.
  *
+ * A session policy is a policy that is permanent to the lifetime of
+ * @hsts_enforcer's [class@Session] and doesn't expire.
  **/
 void
 soup_hsts_enforcer_set_session_policy (SoupHSTSEnforcer *hsts_enforcer,
@@ -426,19 +417,28 @@ static gboolean
 soup_hsts_enforcer_must_enforce_secure_transport (SoupHSTSEnforcer *hsts_enforcer,
 						  const char *domain)
 {
+        SoupHSTSEnforcerPrivate *priv = soup_hsts_enforcer_get_instance_private (hsts_enforcer);
 	const char *super_domain = domain;
 
 	g_return_val_if_fail (domain != NULL, FALSE);
 
-	if (soup_hsts_enforcer_has_valid_policy (hsts_enforcer, domain))
+        g_mutex_lock (&priv->mutex);
+
+	if (soup_hsts_enforcer_has_valid_policy (hsts_enforcer, domain)) {
+                g_mutex_unlock (&priv->mutex);
 		return TRUE;
+        }
 
 	while ((super_domain = super_domain_of (super_domain)) != NULL) {
 		if (soup_hsts_enforcer_host_includes_subdomains (hsts_enforcer, super_domain) &&
-		    soup_hsts_enforcer_has_valid_policy (hsts_enforcer, super_domain))
+		    soup_hsts_enforcer_has_valid_policy (hsts_enforcer, super_domain)) {
+                        g_mutex_unlock (&priv->mutex);
 			return TRUE;
+                }
 	}
 
+        g_mutex_unlock (&priv->mutex);
+
 	return FALSE;
 }
 
@@ -581,7 +581,6 @@ soup_hsts_enforcer_session_feature_init (SoupSessionFeatureInterface *feature_in
  * Gets whether @hsts_enforcer stores policies persistenly.
  *
  * Returns: %TRUE if @hsts_enforcer storage is persistent or %FALSE otherwise.
- *
  **/
 gboolean
 soup_hsts_enforcer_is_persistent (SoupHSTSEnforcer *hsts_enforcer)
@@ -599,8 +598,7 @@ soup_hsts_enforcer_is_persistent (SoupHSTSEnforcer *hsts_enforcer)
  * Gets whether @hsts_enforcer has a currently valid policy for @domain.
  *
  * Returns: %TRUE if access to @domain should happen over HTTPS, false
- * otherwise.
- *
+ *   otherwise.
  **/
 gboolean
 soup_hsts_enforcer_has_valid_policy (SoupHSTSEnforcer *hsts_enforcer,
@@ -641,10 +639,9 @@ add_domain_to_list (gpointer key,
  *
  * Gets a list of domains for which there are policies in @enforcer.
  *
- *
  * Returns: (element-type utf8) (transfer full): a newly allocated
- * list of domains. Use g_list_free_full() and g_free() to free the
- * list.
+ *   list of domains. Use [func@GLib.List.free_full] and [func@GLib.free] to free the
+ *   list.
  **/
 GList*
 soup_hsts_enforcer_get_domains (SoupHSTSEnforcer *hsts_enforcer,
@@ -679,10 +676,8 @@ add_policy_to_list (gpointer key,
  * Gets a list with the policies in @enforcer.
  *
  * Returns: (element-type SoupHSTSPolicy) (transfer full): a newly
- * allocated list of policies. Use g_list_free_full() and
- * soup_hsts_policy_free() to free the list.
- *
- *
+ *   allocated list of policies. Use [func@GLib.List.free_full] and
+ *   [method@HSTSPolicy.free] to free the list.
  **/
 GList*
 soup_hsts_enforcer_get_policies (SoupHSTSEnforcer *hsts_enforcer,
