@@ -61,6 +61,7 @@ soup_add_completion_reffed (GMainContext   *async_context,
 {
 	GSource *source = g_idle_source_new ();
 
+	g_source_set_static_name (source, "SoupCompletion");
 	g_source_set_priority (source, G_PRIORITY_DEFAULT);
 	g_source_set_callback (source, function, data, dnotify);
 	g_source_attach (source, async_context);
@@ -109,6 +110,7 @@ soup_add_timeout (GMainContext *async_context,
 		  GSourceFunc function, gpointer data)
 {
 	GSource *source = g_timeout_source_new (interval);
+	g_source_set_static_name (source, "SoupTimeout");
 	g_source_set_callback (source, function, data, NULL);
 	g_source_attach (source, async_context);
 	return source;
