@@ -180,6 +180,7 @@ static gboolean
 timeout_finish_message (gpointer msg)
 {
 	soup_server_message_unpause (msg);
+        g_object_unref (msg);
 	return FALSE;
 }
 
@@ -199,7 +200,7 @@ server_handler (SoupServer        *server,
                 GSource *timeout;
 		soup_server_message_pause (msg);
 		timeout = soup_add_timeout (g_main_context_get_thread_default (),
-                                            4000, timeout_finish_message, msg);
+                                            4000, timeout_finish_message, g_object_ref (msg));
                 g_source_unref (timeout);
 	}
 }
